<?php

namespace Drupal\graphql_export\Commands;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Site\Settings;
use Drupal\graphql\GraphQL\Utility\Introspection;
use Drush\Commands\DrushCommands;
use Consolidation\AnnotatedCommand\CommandData;

/**
 * Adds the post-command hook to the config:export command.
 */
class GraphqlExportCommands extends DrushCommands {

  public function __construct(Introspection $introspectionService) {
    $this->introspectionService = $introspectionService;
  }

  /**
   * @var \Drupal\graphql\GraphQL\Utility\Introspection
   */
  protected $introspectionService;

  /**
   * @hook post-command config:export
   */
  public function postCommand($result, CommandData $commandData) {
    // First, we check if the module is configured. If not, we log a message.
    $settings = Settings::get('graphql_export', []);
    if (empty($settings)) {
      $this->logger()->warning('The graphql_export module has not been configured. Therefore, the GraphQL schema has not been exported.');
      return;
    }

    // Now, we extract the configuration.
    if (empty($settings['file'])) {
      throw new \Exception('The graphql_export module has been configured, but no result file destination has been provided.');
    }
    $settings_file = $settings['file'];
    $settings_file_folder = dirname($settings_file);

    if (!is_dir($settings_file_folder)) {
      throw new \Exception("The graphql_export is configured to write to $settings_file, but $settings_file_folder doesn't exist or is not a folder.");
    }
    if (!is_writable($settings_file_folder)) {
      throw new \Exception("The graphql_export is configured to write to $settings_file, but $settings_file_folder isn't writable.");
    }
    // The file isn't writeable if the file already exists but cannot be deleted.
    if (is_file($settings_file) && !unlink($settings_file)) {
      throw new \Exception("The graphql_export is configured to write to $settings_file, but the file isn't writable.");
    }

    $settings_schema = !empty($settings['schema']) ? $settings['schema'] : 'default:default';

    $introspectionResult = $this->introspectionService->introspect($settings_schema);

    $introspectionResultJson = Json::encode($introspectionResult);

    if (!file_put_contents($settings_file, $introspectionResultJson)) {
      throw new \Exception("The graphql_export is configured to write to $settings_file, but the result of the introspection could not be written to that file.");
    }

    $this->logger()->notice('The graphql_export module has exported the GraphQL schema.');
  }

}
